/////////////////////////////////////////////////////////////////////////
// Bandwidth functions

export class Bandwidth {

    constructor() {
        this.count = BigInt(0)
        this.rate = BigInt(1)
        this.rateCost = BigInt(321)
    }

    // save data
    save() {
        let save_data = {}
        let props = [
            'count',
            'rate', 'rateCost'
        ]

        props.forEach((prop) => {
            save_data[prop] = this[prop]
        })
         return save_data
    }

    restore(save_data) {
        Object.assign(this, save_data)
    }

    // handle upgrades
    canUpgrade(state) {
        return this.rateCost <= state.levels[0].count
    }

    upgrade(state) {
        // use up packets
        state.levels[0].count -= this.rateCost

        // increase bandwidth rate and cost
        if (this.rate < 10) {
            this.rate += 1n
            this.rateCost *= 2n
        }
        else {
            this.rate *= 2n
            this.rateCost += this.rateCost * this.rate
        }
    }

    serviceBonus(state) {
        let bonus = 0n

        if (state.services.time.level > 0) {
            let i 
            for(i = state.levels.length - 1; i >= 0; i--) {
                if (state.levels[i].unlocked) {
                    let multiplier = state.ui.elapsed - state.levels[i].unlockTime
                    // multiplier = multiplier > 7200 ? 7200 : multiplier
                    bonus += (this.rate * BigInt(multiplier)) / BigInt(Math.floor(7200 / i))
                    break
                }
            }
            if (state.services.time.level > 1) {
                //let multiplier = state.ui.elapsed > 172800 ? 86400 : state.ui.elapsed
                let multiplier = state.ui.elapsed
                bonus += (this.rate * BigInt(multiplier)) / BigInt(Math.floor(86400/i))
            }    
        }

        return bonus
    }

    // regular updates
    tick(state, interval) {
        this.count += this.rate * interval + this.serviceBonus(state)
    }

}

/////////////////////////////////////////////////////////////////////////
// end of file